"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdapterError = void 0;
const InternalError_cjs_1 = require("./InternalError.cjs");
const NameofError_cjs_1 = require("./NameofError.cjs");
/**
 * Represents an error that occurred in an adapter.
 *
 * @template TInput
 * The type of the input of the adapter.
 *
 * @template TOutput
 * The type of the output of the adapter.
 */
class AdapterError extends InternalError_cjs_1.InternalError {
    /**
     * Initializes a new instance of the {@linkcode AdapterError} class.
     *
     * @param adapter
     * The adapter which caused the error.
     *
     * @param node
     * The node related to the error.
     *
     * @param context
     * The context of the operation.
     */
    constructor(adapter, node, context) {
        super();
        this.adapter = adapter;
        this.node = node;
        this.context = context;
    }
    /**
     * @inheritdoc
     */
    get ReportAction() {
        return () => this.Report();
    }
    /**
     * Gets the adapter which caused the error.
     */
    get Adapter() {
        return this.adapter;
    }
    /**
     * Gets the node related to the error.
     */
    get Node() {
        return this.node;
    }
    /**
     * Gets the context of the operation.
     */
    get Context() {
        return this.context;
    }
    /**
     * Gets the expected name of nameof function calls.
     */
    get NameofName() {
        return this.Adapter.GetNameofName(this.Context);
    }
    /**
     * Gets the source code which caused the error.
     */
    get SourceCode() {
        return this.Adapter.GetSourceCode(this.Node, this.Context);
    }
    /**
     * Gets an escaped representation of the {@linkcode SourceCode}.
     */
    get EscapedCode() {
        return this.SourceCode.replace(/([\\`])/g, "\\$1");
    }
    /**
     * Reports the error.
     */
    Report() {
        this.Adapter.ReportError(this.Node, this.Context, new NameofError_cjs_1.NameofError(this.constructor.name, this.Message));
    }
}
exports.AdapterError = AdapterError;
//# sourceMappingURL=AdapterError.cjs.map