"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Adapter = void 0;
const ResultBuilder_cjs_1 = require("./ResultBuilder.cjs");
const CustomError_cjs_1 = require("../Diagnostics/CustomError.cjs");
const IndexOutOfBoundsError_cjs_1 = require("../Diagnostics/IndexOutOfBoundsError.cjs");
const IndexParsingError_cjs_1 = require("../Diagnostics/IndexParsingError.cjs");
const InternalError_cjs_1 = require("../Diagnostics/InternalError.cjs");
const InvalidArgumentCountError_cjs_1 = require("../Diagnostics/InvalidArgumentCountError.cjs");
const InvalidDefaultCallError_cjs_1 = require("../Diagnostics/InvalidDefaultCallError.cjs");
const InvalidSegmentCallError_cjs_1 = require("../Diagnostics/InvalidSegmentCallError.cjs");
const MissingPropertyAccessError_cjs_1 = require("../Diagnostics/MissingPropertyAccessError.cjs");
const NestedNameofError_cjs_1 = require("../Diagnostics/NestedNameofError.cjs");
const SegmentNotFoundError_cjs_1 = require("../Diagnostics/SegmentNotFoundError.cjs");
const UnsupportedAccessorTypeError_cjs_1 = require("../Diagnostics/UnsupportedAccessorTypeError.cjs");
const UnsupportedFunctionError_cjs_1 = require("../Diagnostics/UnsupportedFunctionError.cjs");
const UnsupportedNodeError_cjs_1 = require("../Diagnostics/UnsupportedNodeError.cjs");
const UnsupportedScenarioError_cjs_1 = require("../Diagnostics/UnsupportedScenarioError.cjs");
const ResultType_cjs_1 = require("../ResultType.cjs");
const AccessExpressionNode_cjs_1 = require("../Serialization/AccessExpressionNode.cjs");
const InterpolationNode_cjs_1 = require("../Serialization/InterpolationNode.cjs");
const UnsupportedNode_cjs_1 = require("../Serialization/UnsupportedNode.cjs");
/**
 * Provides the functionality to parse and dump `nameof` expressions.
 */
class Adapter {
    /**
     * Initializes a new instance of the {@linkcode Adapter Adapter<TFeatures, TInput, TNode, TContext>} class.
     *
     * @param features
     * The features of the transformer integration.
     */
    constructor(features) {
        /**
         * A symbol for storing original nodes generated nodes.
         */
        this.originalSymbol = Symbol("nameof-original-node");
        this.features = features;
    }
    /**
     * Gets the features of the transformer integration.
     */
    get Features() {
        return this.features;
    }
    /**
     * Gets a symbol for storing original nodes generated nodes.
     */
    get OriginalSymbol() {
        return this.originalSymbol;
    }
    /**
     * @inheritdoc
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The expected name of function calls.
     */
    GetNameofName(context) {
        return "nameof";
    }
    /**
     * @inheritdoc
     *
     * @param item
     * The item to check.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * A value indicating whether specified {@linkcode item} is mutated.
     */
    IsMutated(item, context) {
        return this.GetOriginal(item) ? true : false;
    }
    /**
     * @inheritdoc
     *
     * @param input
     * The item to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The transformed node.
     */
    Transform(input, context) {
        let node = this.Extract(input);
        if (this.IsCallExpression(node) ||
            this.IsAccessExpression(node)) {
            try {
                let newNode = node;
                let nameofCall = this.GetNameofCall(this.Parse(node, context), context);
                if (nameofCall) {
                    try {
                        let result = this.ProcessNameofCall(nameofCall, context);
                        if (result) {
                            if (Array.isArray(result)) {
                                newNode = this.DumpArray(result);
                            }
                            else {
                                newNode = this.Dump(result);
                            }
                            return newNode;
                        }
                    }
                    catch (error) {
                        throw error;
                    }
                    finally {
                        this.StoreOriginal(node, newNode);
                    }
                }
            }
            catch (error) {
                if (error instanceof InternalError_cjs_1.InternalError) {
                    error.ReportAction();
                }
                else {
                    let newError;
                    if (error instanceof Error) {
                        newError = error;
                    }
                    else {
                        newError = new Error(`${error}`);
                    }
                    this.ReportError(node, context, newError);
                }
            }
        }
        return node;
    }
    /**
     * @inheritdoc
     *
     * @param item
     * The item related to the error.
     *
     * @param context
     * The context of the operation.
     *
     * @param error
     * The error to handle.
     */
    ReportError(item, context, error) {
        this.Features.ReportError(this.GetLocation(item, context), item, context, error);
    }
    /**
     * Stores the specified {@linkcode original} node somewhere in the {@linkcode newNode}.
     *
     * @param original
     * The node to store.
     *
     * @param newNode
     * The newly created node.
     */
    StoreOriginal(original, newNode) {
        newNode[this.OriginalSymbol] = original;
    }
    /**
     * Gets the original node which was substituted by the specified {@linkcode node}.
     *
     * @param node
     * The node to get the original from.
     *
     * @returns
     * The original node of the specified {@linkcode node}.
     */
    GetOriginal(node) {
        return node[this.OriginalSymbol];
    }
    /**
     * Gets the {@linkcode NameofCall} represented by the specified {@linkcode node}.
     *
     * @param node
     * The node to convert.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The either a {@linkcode NameofCall} or `undefined` if no `nameof` call was found.
     */
    GetNameofCall(node, context) {
        if (node instanceof AccessExpressionNode_cjs_1.AccessExpressionNode) {
            let nameofCall = this.GetNameofCall(node.Expression, context);
            if (nameofCall &&
                nameofCall.function === "typed" /* NameofFunction.Typed */) {
                return Object.assign(Object.assign({}, nameofCall), { source: node.Source, typeArguments: [], arguments: [
                        node.Source
                    ] });
            }
        }
        else if (node.Type === "CallExpressionNode" /* NodeKind.CallExpressionNode */) {
            let expression = this.Parse(node.Expression, context);
            let property;
            if (expression.Type === "PropertyAccessNode" /* NodeKind.PropertyAccessNode */) {
                property = expression.PropertyName;
                expression = expression.Expression;
            }
            else if (expression.Type === "IndexAccessNode" /* NodeKind.IndexAccessNode */ &&
                expression.Property.Type === "StringLiteralNode" /* NodeKind.StringLiteralNode */) {
                property = expression.Property.Text;
                expression = expression.Expression;
            }
            if (expression.Type === "IdentifierNode" /* NodeKind.IdentifierNode */ &&
                expression.Name === this.GetNameofName(context)) {
                return {
                    source: node.Source,
                    function: property,
                    typeArguments: node.TypeArguments,
                    arguments: node.Arguments
                };
            }
        }
        else if (node.Type === "InterpolationNode" /* NodeKind.InterpolationNode */) {
            return {
                source: node.Source,
                function: "interpolate" /* NameofFunction.Interpolate */,
                arguments: node.Arguments,
                typeArguments: node.TypeArguments
            };
        }
        return undefined;
    }
    /**
     * Gets the targets of the specified `nameof` {@linkcode call}.
     *
     * @param call
     * The call to get the targets from.
     *
     * @returns
     * The targets of the specified {@linkcode call}.
     */
    GetTargets(call) {
        if (call.arguments.length > 0) {
            return call.arguments;
        }
        else {
            return call.typeArguments;
        }
    }
    /**
     * Parses the specified {@linkcode item}.
     *
     * @param item
     * The item to parse.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode item}.
     */
    Parse(item, context) {
        let node = this.TryParse(item, context);
        if (this.IsCallExpression(item)) {
            let nameofCall = this.GetNameofCall(node, context);
            if (nameofCall &&
                nameofCall.function === "interpolate" /* NameofFunction.Interpolate */) {
                let expectedLength = 1;
                if (nameofCall.arguments.length === expectedLength) {
                    return new InterpolationNode_cjs_1.InterpolationNode(nameofCall.source, nameofCall.arguments[0], nameofCall.typeArguments, nameofCall.arguments);
                }
                else {
                    throw new InvalidArgumentCountError_cjs_1.InvalidArgumentCountError(this, nameofCall, expectedLength, context);
                }
            }
        }
        return node;
    }
    /**
     * Tries to parse the specified {@linkcode item} and returns the result or the error that occurred.
     *
     * @param item
     * The item to parse.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode item} or a {@linkcode UnsupportedNode} in case a documented error occurred.
     */
    TryParse(item, context) {
        try {
            return this.ParseInternal(item, context);
        }
        catch (error) {
            if (error instanceof InternalError_cjs_1.InternalError) {
                return new UnsupportedNode_cjs_1.UnsupportedNode(item, error);
            }
            else {
                throw error;
            }
        }
    }
    /**
     * Transforms the specified `nameof` {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The transformed call.
     */
    ProcessNameofCall(call, context) {
        switch (call.function) {
            case undefined:
                return this.ProcessDefault(call, context);
            case "typed" /* NameofFunction.Typed */:
                return this.ProcessTyped(call, context);
            case "full" /* NameofFunction.Full */:
                return this.ProcessFull(call, context);
            case "split" /* NameofFunction.Split */:
                return this.ProcessSplit(call, context);
            case "array" /* NameofFunction.Array */:
            case "toArray" /* NameofFunction.LegacyArray */:
                return this.ProcessArray(call, context);
            case "interpolate" /* NameofFunction.Interpolate */:
                return this.ProcessInterpolate(call, context);
            default:
                throw new UnsupportedFunctionError_cjs_1.UnsupportedFunctionError(this, call, context);
        }
    }
    /**
     * Processes the specified `nameof` {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode call}.
     */
    ProcessDefault(call, context) {
        let targets = this.GetTargets(call);
        if (targets.length === 1) {
            return this.GetName(call, this.ProcessSingle(call, targets[0], context), context);
        }
        else {
            throw new InvalidDefaultCallError_cjs_1.InvalidDefaultCallError(this, call, context);
        }
    }
    /**
     * Processes the specified `nameof.typed` {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode call}.
     */
    ProcessTyped(call, context) {
        var _a, _b, _c, _d;
        if (this.IsAccessExpression(call.source)) {
            let accessExpression = this.ParseInternal(call.source, context);
            let index = (_b = (_a = context.typedCalls) === null || _a === void 0 ? void 0 : _a.indexOf(accessExpression.Expression.Source)) !== null && _b !== void 0 ? _b : -1;
            if (index >= 0) {
                (_c = context.typedCalls) === null || _c === void 0 ? void 0 : _c.splice(index, 1);
            }
            return this.ProcessDefault(call, context);
        }
        else {
            (_d = context.typedCalls) !== null && _d !== void 0 ? _d : (context.typedCalls = []);
            context.typedCalls.push(call.source);
            return undefined;
        }
    }
    /**
     * Processes the specified `nameof.full` {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode call}.
     */
    ProcessFull(call, context) {
        let path = this.ProcessSegment(call, context);
        let resultBuilder = new ResultBuilder_cjs_1.ResultBuilder(this, context);
        for (let pathPart of path) {
            resultBuilder.Add(pathPart);
        }
        return resultBuilder.Result;
    }
    /**
     * Processes the specified `nameof.split` {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode call}.
     */
    ProcessSplit(call, context) {
        let path = this.ProcessSegment(call, context);
        let result = [];
        for (let pathPart of path) {
            switch (pathPart.type) {
                case "Interpolation" /* PathKind.Interpolation */:
                    throw new UnsupportedScenarioError_cjs_1.UnsupportedScenarioError(this, pathPart.source, context);
                default:
                    result.push({
                        type: ResultType_cjs_1.ResultType.Plain,
                        text: `${pathPart.value}`
                    });
            }
        }
        return result;
    }
    /**
     * Processes the specified `nameof.toArray` {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode call}.
     */
    ProcessArray(call, context) {
        let expressions = call.arguments;
        let processor = (node) => {
            return this.GetName(call, this.Parse(node, context).Path, context);
        };
        if (call.arguments.length === 1) {
            let parsedNode = this.Parse(call.arguments[0], context);
            if (parsedNode.Type === "FunctionNode" /* NodeKind.FunctionNode */) {
                let functionNode = parsedNode;
                let elements = this.GetArrayElements(parsedNode.Body);
                if (elements) {
                    expressions = elements;
                    processor = (node) => {
                        return this.GetName(call, this.ProcessFunctionBody(functionNode, node, context), context);
                    };
                }
            }
        }
        return expressions.map((expression) => {
            if (this.IsStringLiteral(expression) ||
                this.IsTemplateLiteral(expression)) {
                if (this.IsMutated(expression, context)) {
                    return {
                        type: ResultType_cjs_1.ResultType.Node,
                        node: expression
                    };
                }
                else {
                    throw new UnsupportedNodeError_cjs_1.UnsupportedNodeError(this, expression, context);
                }
            }
            else {
                return processor(expression);
            }
        });
    }
    /**
     * Processes the specified `nameof.interpolate` {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The transformed call.
     */
    ProcessInterpolate(call, context) {
        var _a;
        // Don't transform `interpolate` functions - they will be processed in the `ProcessFull` method.
        (_a = context.interpolationCalls) !== null && _a !== void 0 ? _a : (context.interpolationCalls = []);
        context.interpolationCalls.push(call.source);
        return;
    }
    /**
     * Processes a segment of the specified {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The transformed call.
     */
    ProcessSegment(call, context) {
        let index;
        let expression;
        if (call.arguments.length === 0) {
            if (call.typeArguments.length === 1) {
                expression = call.typeArguments[0];
            }
            else {
                throw new InvalidSegmentCallError_cjs_1.InvalidSegmentCallError(this, call, context);
            }
        }
        else if (call.arguments.length === 1) {
            if (call.typeArguments.length === 0) {
                expression = call.arguments[0];
            }
            else if (call.typeArguments.length === 1) {
                let parsedArgument = this.Parse(call.arguments[0], context);
                if (parsedArgument.Type === "NumericLiteralNode" /* NodeKind.NumericLiteralNode */) {
                    index = parsedArgument;
                    expression = call.typeArguments[0];
                }
                else {
                    expression = call.arguments[0];
                }
            }
            else {
                throw new InvalidSegmentCallError_cjs_1.InvalidSegmentCallError(this, call, context);
            }
        }
        else if (call.arguments.length === 2) {
            let parsedNode = this.Parse(call.arguments[1], context);
            if (parsedNode.Type === "NumericLiteralNode" /* NodeKind.NumericLiteralNode */) {
                index = parsedNode;
                expression = call.arguments[0];
            }
            else {
                throw new IndexParsingError_cjs_1.IndexParsingError(this, parsedNode.Source, context);
            }
        }
        else {
            throw new InvalidSegmentCallError_cjs_1.InvalidSegmentCallError(this, call, context);
        }
        let path = this.ProcessSingle(call, expression, context);
        if (index) {
            return this.GetPath(call, path, index, context);
        }
        else {
            return this.GetPathSegments(call, path, 0, path.length, context);
        }
    }
    /**
     * Processes the specified {@linkcode call}.
     *
     * @param call
     * The call to transform.
     *
     * @param node
     * The node to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The transformed call.
     */
    ProcessSingle(call, node, context) {
        let result = this.Parse(node, context);
        if (result.Type === "FunctionNode" /* NodeKind.FunctionNode */) {
            return this.ProcessFunctionBody(result, result.Body, context);
        }
        else {
            return result.Path;
        }
    }
    /**
     * Processes the specified {@linkcode node}.
     *
     * @param functionNode
     * The function of the node to transform.
     *
     * @param node
     * The node to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The transformed representation of the specified {@linkcode node}.
     */
    ProcessFunctionBody(functionNode, node, context) {
        let path = this.Parse(node, context).Path;
        if (path[0].type === "Identifier" /* PathKind.Identifier */ &&
            functionNode.Parameters.includes(path[0].value)) {
            path.splice(0, 1);
            if (path.length === 0) {
                throw new MissingPropertyAccessError_cjs_1.MissingPropertyAccessError(this, node, context);
            }
        }
        return path;
    }
    /**
     * Gets the trailing name of the specified {@linkcode path}.
     *
     * @param call
     * The call which is being transformed.
     *
     * @param path
     * The path to get the name from.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The trailing name of the specified {@linkcode path}.
     */
    GetName(call, path, context) {
        try {
            let lastNode = this.GetPathSegments(call, path, path.length - 1, 1, context)[0];
            switch (lastNode.type) {
                case "Identifier" /* PathKind.Identifier */:
                case "PropertyAccess" /* PathKind.PropertyAccess */:
                case "IndexAccess" /* PathKind.IndexAccess */:
                    return {
                        type: ResultType_cjs_1.ResultType.Plain,
                        text: `${lastNode.value}`
                    };
                case "Interpolation" /* PathKind.Interpolation */:
                    throw new UnsupportedScenarioError_cjs_1.UnsupportedScenarioError(this, lastNode.source, context);
            }
        }
        catch (error) {
            if (error instanceof SegmentNotFoundError_cjs_1.SegmentNotFoundError) {
                throw new CustomError_cjs_1.CustomError(this, call.source, context, "Unable to find an expression to get the name from.");
            }
            else {
                throw error;
            }
        }
    }
    /**
     * Extracts a portion of the specified {@linkcode path} according to the specified {@linkcode index}.
     *
     * @param call
     * The call which is being transformed.
     *
     * @param path
     * The path to extract the portion from.
     *
     * @param index
     * The index of the element to start the extraction from.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * A portion of the specified {@linkcode path} according to the specified {@linkcode index}.
     */
    GetPath(call, path, index, context) {
        if (index.Value === path.length ||
            Math.abs(index.Value) > path.length) {
            throw new IndexOutOfBoundsError_cjs_1.IndexOutOfBoundsError(this, index, path.length, context);
        }
        else {
            let startIndex;
            let count;
            if (index.Value >= 0) {
                startIndex = index.Value;
                count = path.length - startIndex;
            }
            else {
                startIndex = path.length + index.Value;
                count = -index.Value;
            }
            return this.GetPathSegments(call, path, startIndex, count, context);
        }
    }
    /**
     * Gets the specified portion from the specified {@linkcode path}.
     *
     * @param call
     * The call which is being transformed.
     *
     * @param path
     * The path to extract the specified portion from.
     *
     * @param start
     * The index of the item to start extracting the path.
     *
     * @param count
     * The number if items to get.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The specified portion of the specified {@linkcode path}.
     */
    GetPathSegments(call, path, start, count, context) {
        if (start < path.length) {
            let result = [];
            let candidates = path.slice(start);
            if (candidates.length >= count) {
                candidates = candidates.slice(0, count);
                for (let pathPart of candidates) {
                    if (pathPart.type === "Unsupported" /* PathKind.Unsupported */) {
                        if (pathPart.reason) {
                            throw pathPart.reason;
                        }
                        else if (pathPart.isAccessor) {
                            throw new UnsupportedAccessorTypeError_cjs_1.UnsupportedAccessorTypeError(this, pathPart.source, context);
                        }
                        else {
                            let original = this.GetOriginal(pathPart.source);
                            if (original) {
                                throw new NestedNameofError_cjs_1.NestedNameofError(this, original, context);
                            }
                            else {
                                throw new UnsupportedNodeError_cjs_1.UnsupportedNodeError(this, pathPart.source, context);
                            }
                        }
                    }
                    else {
                        result.push(pathPart);
                    }
                }
                return result;
            }
        }
        throw new SegmentNotFoundError_cjs_1.SegmentNotFoundError(this, call, context);
    }
    /**
     * Dumps the specified {@linkcode items}.
     *
     * @param items
     * The items to dump-
     *
     * @returns
     * The newly created node.
     */
    DumpArray(items) {
        return this.CreateArrayLiteral(items.map((item) => this.Dump(item)));
    }
}
exports.Adapter = Adapter;
//# sourceMappingURL=Adapter.cjs.map