"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResultBuilder = void 0;
const ResultType_cjs_1 = require("../ResultType.cjs");
/**
 * Provides the functionality to reduce a path to a single result.
 */
class ResultBuilder {
    /**
     * Initializes a new instance of the {@linkcode ResultBuilder} class.
     *
     * @param adapter
     * The adapter of the result builder.
     *
     * @param context
     * The context of the operation.
     */
    constructor(adapter, context) {
        /**
         * The template parts of the result.
         */
        this.templateParts = [];
        /**
         * The expressions of the result.
         */
        this.expressions = [];
        /**
         * Gets the current template part of the result.
         */
        this.current = "";
        this.adapter = adapter;
        this.context = context;
    }
    /**
     * Gets the built result.
     */
    get Result() {
        if (this.Expressions.length > 0) {
            return {
                type: ResultType_cjs_1.ResultType.Template,
                templateParts: [...this.TemplateParts, this.Current],
                expressions: this.Expressions
            };
        }
        else {
            return {
                type: ResultType_cjs_1.ResultType.Plain,
                text: this.Current
            };
        }
    }
    /**
     * Gets the adapter of the result builder.
     */
    get Adapter() {
        return this.adapter;
    }
    /**
     * Gets the context of the operation.
     */
    get Context() {
        return this.context;
    }
    /**
     * Gets a value indicating whether the current state is empty.
     */
    get Empty() {
        return this.Current.length === 0 && this.TemplateParts.length === 0;
    }
    /**
     * Gets the template parts of the result.
     */
    get TemplateParts() {
        return this.templateParts;
    }
    /**
     * Gets the expressions of the result.
     */
    get Expressions() {
        return this.expressions;
    }
    /**
     * Gets or sets the current template part of the result.
     */
    get Current() {
        return this.current;
    }
    /**
     * @inheritdoc
     */
    set Current(value) {
        this.current = value;
    }
    /**
     * Adds the specified {@linkcode pathPart} to the result.
     *
     * @param pathPart
     * The part to add to the result.
     */
    Add(pathPart) {
        var _a, _b, _c;
        switch (pathPart.type) {
            case "Identifier" /* PathKind.Identifier */:
            case "PropertyAccess" /* PathKind.PropertyAccess */:
                if (this.Empty) {
                    this.Current = pathPart.value;
                }
                else {
                    this.Current += `.${pathPart.value}`;
                }
                break;
            case "IndexAccess" /* PathKind.IndexAccess */:
                this.Current += `[${JSON.stringify(pathPart.value)}]`;
                break;
            case "Interpolation" /* PathKind.Interpolation */:
                let index = (_b = (_a = this.Context.interpolationCalls) === null || _a === void 0 ? void 0 : _a.indexOf(pathPart.source)) !== null && _b !== void 0 ? _b : -1;
                if (index >= 0) {
                    (_c = this.Context.interpolationCalls) === null || _c === void 0 ? void 0 : _c.splice(index, 1);
                }
                this.Push(pathPart.node);
                break;
        }
    }
    /**
     * Pushes the specified expression on the expression stack.
     *
     * @param expression
     * The expression to add.
     */
    Push(expression) {
        this.Current += "[";
        this.TemplateParts.push(this.Current);
        this.Current = "]";
        this.Expressions.push(expression);
    }
}
exports.ResultBuilder = ResultBuilder;
//# sourceMappingURL=ResultBuilder.cjs.map