"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformerBase = void 0;
const MissingPropertyAccessError_cjs_1 = require("../Diagnostics/MissingPropertyAccessError.cjs");
const UnusedInterpolationError_cjs_1 = require("../Diagnostics/UnusedInterpolationError.cjs");
/**
 * Provides a basic implementation of a transformer.
 *
 * @template TNode
 * The type of the nodes to transform.
 *
 * @template TFeatures
 * The type of the features
 */
class TransformerBase {
    /**
     * Initializes a new instance of the {@linkcode TransformerBase TransformerBase<TNode, TFeatures>} class.
     *
     * @param features
     * A set of features for performing transformations.
     */
    constructor(features) {
        this.features = features;
    }
    /**
     * Gets the adapter for transforming nodes.
     */
    get Adapter() {
        if (!this.adapter) {
            this.adapter = this.InitializeAdapter();
        }
        return this.adapter;
    }
    /**
     * Gets a set of features for performing transformations.
     */
    get Features() {
        return this.features;
    }
    /**
     * Monitors the transformation in the specified {@linkcode action} for errors.
     *
     * @param action
     * The action to monitor.
     *
     * @returns
     * The result of the action.
     */
    MonitorTransformation(action) {
        var _a, _b;
        let context = {
            interpolationCalls: []
        };
        let result = action(context);
        let remainingInterpolationCall = (_a = context.interpolationCalls) === null || _a === void 0 ? void 0 : _a[0];
        let remainingTypedCall = (_b = context.typedCalls) === null || _b === void 0 ? void 0 : _b[0];
        if (remainingInterpolationCall) {
            new UnusedInterpolationError_cjs_1.UnusedInterpolationError(this.Adapter, remainingInterpolationCall, context).ReportAction();
        }
        if (remainingTypedCall) {
            new MissingPropertyAccessError_cjs_1.MissingPropertyAccessError(this.Adapter, remainingTypedCall, context).ReportAction();
        }
        return result;
    }
}
exports.TransformerBase = TransformerBase;
//# sourceMappingURL=TransformerBase.cjs.map